
/*****************************************************************************
* 본 예제는 ModbusRTU 무료 라이브러리를 이용하여 
* DTPML-485 의 온도 데이터를 읽어오는 예제 코드 입니다.	
* 본 코드에서 사용하는 라이브러리는 개인, 업체 또는 특정 목적으로의 활용에 있어서
* 무결점을 보증하지 않습니다.

* Project Name : (DTPML-485시리즈) RS485 Modbus-RTU Master Code
* Version : 1.0 
* SYSTEM CLOCK : 16Mhz 
* BOARD : Arduino UNO. 5V operation 

 PORT Description

1. RX : 3            
2. TX : 5	     
3. DE/RE : 2

* 회로 연결 방법은 첨부 pdf 를 참고 하십시오.
* 업로드 전에 Modbus 라이브러리를 설치해야 합니다.(압축 파일)
* 485 IC 없이 모듈과 아두이노를 연결하면 절대 안됩니다.
  IO 포트 및 IC 손상이 발생할 수 있습니다.

 Revision history.

1.0 2020.1.2  : First version is released.

****************************************************************************/

#include <ModbusRtu.h>
#include "SoftwareSerial.h"
#define DISABLE  0
#define ENABLE   1
#define DE_RE  2
#define USING_SOFTWARESERIAL  4
#define ID  1

uint8_t Timer1_Flag=0, Data_Print = DISABLE;
uint16_t au16data[2];
int32_t Sensor, Object;

SoftwareSerial mySerial(3, 5);                      // RX 3, TX 5
Modbus master(0,USING_SOFTWARESERIAL,DE_RE);          // Modbus Master, 소프트웨어시리얼사용, DE/RE 제어 포트 2번
modbus_t telegram;                                   // Master query structure

void setup() {
  pinMode(DE_RE,OUTPUT);                              // DE_RE output 
  Serial.begin(9600);                                // use the hardware serial if you want to connect to your computer via usb cable.
  master.begin(&mySerial, 19200);                     // begin the ModBus object.
  master.setTimeOut( 20 );                            // if there is no answer in 20 ms, roll over
  
  delay(200);                                         // Wait Sensor init (minimum 200ms)
  Timer1_Init();                                      // Timer1 setup : 100ms interval
  interrupts();                                       // enable all interrupts
}

void loop(){
  if(Timer1_Flag)                                    // 100ms 마다 요청 데이터 전송
  {
    Timer1_Flag=0;
    Transfer_Data(ID);                                 // TX
  }
  if(master.getState() == COM_WAITING)               
  {
   master.poll();                                    // 데이터 수신
   Data_Print = ENABLE;                              // 데이터 반복 출력 방지 플레그
  }
    if ((master.getState() == COM_IDLE) && (Data_Print == ENABLE))       // 통신 상태 : idle
    {
         if(master.getLastError() == 0 )                 // 통신에 문제 없으면 0 리턴
         {
           Object = au16data[0];                          //4 byte 변수에 저장
           Sensor = au16data[1];                       
          
           Serial.print("Object Temp : ");
           Serial.print(float(Object)/10, 1);                 // 실수형으로 변환
           Serial.print("    Sensor Temp : ");
           Serial.println(float(Sensor)/10, 1);               // 실수형으로 변환
         }
         else if(master.getLastError() == NO_REPLY)          // 응답이 없을 경우. 회로 연결에 문제가 없는지 체크하세요.
         {  
            Serial.println("No reply.");
         }
         Data_Print = DISABLE;                               // 데이터 반복 출력 방지 플레그
      }
   
}

ISR(TIMER1_OVF_vect)                                  // interrupt service routine : interval 100ms
{
  TCNT1 = 59286;                                      //  65536-16MHz/256/10Hz = 65536 - 16000000/256/10Hz = 59286
  Timer1_Flag = 1;
}
void Timer1_Init(void)
{
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 59286;            // preload timer 65536-16MHz/256/10Hz = 65536 - 16000000/256/10 = 59286
  TCCR1B |= (1 << CS12);    // 256 prescaler 
  TIMSK1 |= (1 << TOIE1);   // enable timer overflow interrupt
}
void Transfer_Data(uint8_t uid)
{
  if( (uid == 0) &&(uid>200) )                      // ID : 1~200 
  {
    uid = 1;                                        // 범위 오류시 ID 1로 강제 설정
  }
  
   telegram.u8id = uid;                             // slave ID 
   telegram.u8fct = 3;                              // function code = 03 (Read Holding Registers) 
   telegram.u16RegAdd = 1200;                       // start address in slave 
   telegram.u16CoilsNo = 2;                         // number of elements to read : Only 2 support.
   telegram.au16reg = au16data;                     // pointer to a memory array in the Arduino 

   master.query( telegram );                        // send query
}
